@@ -2,16 +2,16 @@
  * BRLTTY - A background process providing access to the console screen (when in
  *          text mode) for a blind person using a refreshable braille display.
  *
- * Copyright (C) 1995-2011 by The BRLTTY Developers.
+ * Copyright (C) 1995-2023 by The BRLTTY Developers.
  *
  * BRLTTY comes with ABSOLUTELY NO WARRANTY.
  *
  * This is free software, placed under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option) any
- * later version. Please see the file LICENSE-GPL for details.
+ * GNU Lesser General Public License, as published by the Free Software
+ * Foundation; either version 2.1 of the License, or (at your option) any
+ * later version. Please see the file LICENSE-LGPL for details.
  *
- * Web Page: http://mielke.cc/brltty/
+ * Web Page: http://brltty.app/
  *
  * This software is maintained by Dave Mielke <dave@mielke.cc>.
  */
@@ -28,15 +28,11 @@
 typedef enum {
 	PARM_PATH,
 	PARM_PUNCTLIST,
-	PARM_VOICE
+	PARM_VOICE,
+	PARM_MAXRATE
 } DriverParameter;
-#define SPKPARMS "path", "punctlist", "voice"
+#define SPKPARMS "path", "punctlist", "voice", "maxrate"
 
-#define SPK_HAVE_TRACK
-#define SPK_HAVE_RATE
-#define SPK_HAVE_VOLUME
-#define SPK_HAVE_PITCH
-#define SPK_HAVE_PUNCTUATION
 #include "spk_driver.h"
 
 #include <espeak/speak_lib.h>
@@ -44,25 +40,94 @@ typedef enum {
 #if ESPEAK_API_REVISION < 6
 #define espeakRATE_MINIMUM	80
 #define espeakRATE_MAXIMUM	450
-#endif
+#endif /* ESPEAK_API_REVISION < 6 */
 
-static int IndexPos;
+static int maxrate = espeakRATE_MAXIMUM;
+
+static void
+spk_say(SpeechSynthesizer *spk, const unsigned char *buffer, size_t length, size_t count, const unsigned char *attributes)
+{
+	int result;
+
+	/* add 1 to the length in order to pass along the trailing zero */
+	result = espeak_Synth(buffer, length+1, 0, POS_CHARACTER, 0,
+			espeakCHARS_UTF8, NULL, (void *)spk);
+	if (result != EE_OK)
+		logMessage(LOG_ERR, "eSpeak: Synth() returned error %d", result);
+}
+
+static void
+spk_mute(SpeechSynthesizer *spk)
+{
+	espeak_Cancel();
+}
 
 static int SynthCallback(short *audio, int numsamples, espeak_EVENT *events)
 {
+	SpeechSynthesizer *spk = events->user_data;
+
 	while (events->type != espeakEVENT_LIST_TERMINATED) {
 		if (events->type == espeakEVENT_WORD)
-			IndexPos = events->text_position - 1;
+			tellSpeechLocation(spk, events->text_position - 1);
+		if (events->type == espeakEVENT_MSG_TERMINATED)
+			tellSpeechFinished(spk);
 		events++;
 	}
 	return 0;
 }
-	
+
+static void
+spk_drain(SpeechSynthesizer *spk)
+{
+	espeak_Synchronize();
+}
+
+static void
+spk_setVolume(SpeechSynthesizer *spk, unsigned char setting)
+{
+	int volume = getIntegerSpeechVolume(setting, 50);
+	espeak_SetParameter(espeakVOLUME, volume, 0);
+}
+
+static void
+spk_setRate(SpeechSynthesizer *spk, unsigned char setting)
+{
+	int h_range = (maxrate - espeakRATE_MINIMUM)/2;
+	int rate = getIntegerSpeechRate(setting, h_range) + espeakRATE_MINIMUM;
+	espeak_SetParameter(espeakRATE, rate, 0);
+}
+
+static void
+spk_setPitch(SpeechSynthesizer *spk, unsigned char setting)
+{
+	int pitch = getIntegerSpeechPitch(setting, 50);
+	espeak_SetParameter(espeakPITCH, pitch, 0);
+}
+
+static void
+spk_setPunctuation(SpeechSynthesizer *spk, SpeechPunctuation setting)
+{
+	espeak_PUNCT_TYPE punct;
+	if (setting <= SPK_PUNCTUATION_NONE)
+		punct = espeakPUNCT_NONE;
+	else if (setting >= SPK_PUNCTUATION_ALL)
+		punct = espeakPUNCT_ALL;
+	else
+		punct = espeakPUNCT_SOME;
+	espeak_SetParameter(espeakPUNCTUATION, punct, 0);
+}
+
 static int spk_construct(SpeechSynthesizer *spk, char **parameters)
 {
-	char *data_path, *voicename, *punctlist;
+	const char *data_path, *voicename, *punctlist;
 	int result;
 
+	spk->setVolume = spk_setVolume;
+	spk->setRate = spk_setRate;
+	spk->setPitch = spk_setPitch;
+	spk->setPunctuation = spk_setPunctuation;
+	spk->drain = spk_drain;
+
 	logMessage(LOG_INFO, "eSpeak version %s", espeak_Info(NULL));
 
 	data_path = parameters[PARM_PATH];
@@ -97,6 +162,11 @@ static int spk_construct(SpeechSynthesizer *spk, char **parameters)
 		espeak_SetPunctuationList(w_punctlist);
 	}
 
+	if (parameters[PARM_MAXRATE]) {
+		int val = atoi(parameters[PARM_MAXRATE]);
+		if (val > espeakRATE_MINIMUM) maxrate = val;
+	}
+
 	espeak_SetSynthCallback(SynthCallback);
 
 	return 1;
@@ -105,78 +175,5 @@ static int spk_construct(SpeechSynthesizer *spk, char **parameters)
 static void spk_destruct(SpeechSynthesizer *spk)
 {
 	espeak_Cancel();
-	espeak_Synchronize();
 	espeak_Terminate();
 }
-
-static void
-spk_say(SpeechSynthesizer *spk, const unsigned char *buffer, size_t length, size_t count, const unsigned char *attributes)
-{
-	int result;
-	
-	IndexPos = 0;
-
-	/* add 1 to the length in order to pass along the trailing zero */
-	result = espeak_Synth(buffer, length+1, 0, POS_CHARACTER, 0,
-			espeakCHARS_UTF8, NULL, NULL);
-	if (result != EE_OK)
-		logMessage(LOG_ERR, "eSpeak: Synth() returned error %d", result);
-}
-
-static void
-spk_mute(SpeechSynthesizer *spk)
-{
-	espeak_Cancel();
-}
-
-static void
-spk_doTrack(SpeechSynthesizer *spk)
-{
-}
-
-static int
-spk_getTrack(SpeechSynthesizer *spk)
-{
-	return IndexPos;
-}
-
-static int
-spk_isSpeaking(SpeechSynthesizer *spk)
-{
-	return espeak_IsPlaying();
-}
-
-static void
-spk_rate(SpeechSynthesizer *spk, unsigned char setting)
-{
-	int h_range = (espeakRATE_MAXIMUM - espeakRATE_MINIMUM)/2;
-	int rate = getIntegerSpeechRate(setting, h_range) + espeakRATE_MINIMUM;
-	espeak_SetParameter(espeakRATE, rate, 0);
-}
-
-static void
-spk_volume(SpeechSynthesizer *spk, unsigned char setting)
-{
-	int volume = getIntegerSpeechVolume(setting, 50);
-	espeak_SetParameter(espeakVOLUME, volume, 0);
-}
-
-static void
-spk_pitch(SpeechSynthesizer *spk, unsigned char setting)
-{
-	int pitch = getIntegerSpeechPitch(setting, 50);
-	espeak_SetParameter(espeakPITCH, pitch, 0);
-}
-
-static void
-spk_punctuation(SpeechSynthesizer *spk, SpeechPunctuation setting)
-{
-	espeak_PUNCT_TYPE punct;
-	if (setting <= SPK_PUNCTUATION_NONE)
-		punct = espeakPUNCT_NONE;
-	else if (setting >= SPK_PUNCTUATION_ALL)
-		punct = espeakPUNCT_ALL;
-	else
-		punct = espeakPUNCT_SOME;
-	espeak_SetParameter(espeakPUNCTUATION, punct, 0);
-}
